package com.yodean.project.gatewayimpl;

import com.alibaba.cola.logger.Logger;
import com.alibaba.cola.logger.LoggerFactory;
import com.yodean.project.common.event.DomainEventPublisher;
import com.yodean.project.domain.gateway.MetricGateway;
import com.yodean.project.domain.metrics.MainMetricType;
import com.yodean.project.domain.metrics.MetricItem;
import com.yodean.project.domain.metrics.SubMetric;
import com.yodean.project.domain.metrics.SubMetricType;
import com.yodean.project.domain.metrics.appquality.AppMetric;
import com.yodean.project.domain.metrics.appquality.AppMetricItem;
import com.yodean.project.domain.metrics.devquality.BugMetric;
import com.yodean.project.domain.metrics.devquality.BugMetricItem;
import com.yodean.project.domain.metrics.techcontribution.*;
import com.yodean.project.domain.metrics.techinfluence.*;
import com.yodean.project.dto.domainevent.MetricItemCreatedEvent;
import com.yodean.project.gatewayimpl.database.MetricMapper;
import com.yodean.project.gatewayimpl.database.dataobject.MetricDO;
import com.yodean.project.gatewayimpl.rpc.AppMetricMapper;
import com.yodean.project.gatewayimpl.rpc.BugMetricMapper;
import com.yodean.project.gatewayimpl.rpc.dataobject.AppMetricDO;
import com.yodean.project.gatewayimpl.rpc.dataobject.BugMetricDO;
import com.yodean.project.convertor.MetricConvertor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * MetricGatewayImpl
 *
 * @author Frank Zhang
 * @date 2020-07-02 12:20 PM
 */
@Component
public class MetricGatewayImpl implements MetricGateway {
    private static Logger logger = LoggerFactory.getLogger(MetricGatewayImpl.class);

    @Resource
    private MetricMapper metricMapper;

    @Resource
    private BugMetricMapper bugMetricMapper;

    @Resource
    private AppMetricMapper appMetricMapper;

    @Resource
    private DomainEventPublisher domainEventPublisher;


    public void save(MetricItem metricItem){
        MetricDO metricDO = MetricConvertor.toDataObject(metricItem);

        metricMapper.create(metricDO);

        logger.debug("AutoGeneratedId: "+metricDO.getId());
        MetricItemCreatedEvent metricItemCreatedEvent = new MetricItemCreatedEvent();
        metricItemCreatedEvent.setId(metricDO.getId());
        metricItemCreatedEvent.setUserId(metricDO.getUserId());
        metricItemCreatedEvent.setMainMetricType(metricDO.getMainMetric());
        domainEventPublisher.publish(metricItemCreatedEvent);
    }


    public List<SubMetric> listByTechContribution(String userId){
        List<MetricDO> metricDOList = metricMapper.listByMainMetric(userId, MainMetricType.TECH_CONTRIBUTION.getMetricCode());
        RefactoringMetric refactoringMetric = new RefactoringMetric();
        MiscMetric miscMetric = new MiscMetric();
        CodeReviewMetric codeReviewMetric = new CodeReviewMetric();
        List<SubMetric> subMetricList = new ArrayList<>();
        subMetricList.add(refactoringMetric);
        subMetricList.add(miscMetric);
        subMetricList.add(codeReviewMetric);
        metricDOList.forEach(metricDO -> {
            String json = metricDO.getMetricItem();
            switch (SubMetricType.valueOf(metricDO.getSubMetric())){
                case Refactoring:
                    refactoringMetric.addMetricItem(RefactoringMetricItem.valueOf(json));
                    break;
                case Misc:
                    miscMetric.addMetricItem(MiscMetricItem.valueOf(json));
                    break;
                case CodeReview:
                    codeReviewMetric.addMetricItem(CodeReviewMetricItem.valueOf(json));
                    break;
                default:
                    logger.error("illegal SubMetric type: " + metricDO.getSubMetric());
            }
        });
        return subMetricList;
    }

    public List<SubMetric> listByTechInfluence(String userId){
        List<MetricDO> metricDOList = metricMapper.listByMainMetric(userId, MainMetricType.TECH_INFLUENCE.getMetricCode());
        ATAMetric ataMetric = new ATAMetric();
        SharingMetric sharingMetric = new SharingMetric();
        PatentMetric patentMetric = new PatentMetric();
        PaperMetric paperMetric = new PaperMetric();
        List<SubMetric> subMetricList = new ArrayList<>();
        subMetricList.add(ataMetric);
        subMetricList.add(sharingMetric);
        subMetricList.add(patentMetric);
        subMetricList.add(paperMetric);
        metricDOList.forEach(metricDO -> {
            String json = metricDO.getMetricItem();
            switch (SubMetricType.valueOf(metricDO.getSubMetric())){
                case ATA:
                    ataMetric.addMetricItem(ATAMetricItem.valueOf(json));
                    break;
                case Sharing:
                    sharingMetric.addMetricItem(SharingMetricItem.valueOf(json));
                    break;
                case Patent:
                    patentMetric.addMetricItem(PatentMetricItem.valueOf(json));
                    break;
                case Paper:
                    paperMetric.addMetricItem(PaperMetricItem.valueOf(json));
                default:
                    logger.error("illegal SubMetric type: " + metricDO.getSubMetric());
            }
        });
        return subMetricList;
    }

    public BugMetric getBugMetric(String userId){
        BugMetricDO bugMetricDO = bugMetricMapper.getByUserId(userId);
        BugMetricItem bugMetricItem = new BugMetricItem(bugMetricDO.getBugCount(), bugMetricDO.getCheckInCodeCount());
        BugMetric bugMetric = new BugMetric();
        bugMetric.addMetricItem(bugMetricItem);
        return bugMetric;
    }

    public AppMetric getAppMetric(String userId){
        List<AppMetricDO> appMetricDOList = appMetricMapper.listByUserId(userId);
        AppMetric appMetric = new AppMetric();
        appMetricDOList.forEach(appMetricDO -> {
            AppMetricItem appMetricItem = new AppMetricItem();
            BeanUtils.copyProperties(appMetricDO, appMetricItem);
            appMetric.addMetricItem(appMetricItem);
        });
        return appMetric;
    }
}
